#include <iostream>
#include <string>
using namespace std;

#include "simpio.h"
#include "random.h"
#include "simpio.h"
#include "gwindow.h"
#include "grid.h"

static const int kBoardDimension = 8;
static const double kSquareSize = 48;
static const double kWindowDimension = (kBoardDimension + 1) * kSquareSize;
static const double kMillisecondDelay = 25;
static const string kQueenLabel = "Q";
static const string kPossiblilityColor = "#aaaaaa";
static const string kProvisionalColor = "#0000ff";
static const string kPermanantColor = "#55aa55";
static const string kNoPossibilityColor = "#ffffff";

static void drawSquare(GWindow& window, int row, int col) {
    int ulx = (window.getWidth() - kBoardDimension * kSquareSize)/2 + col * kSquareSize;
    int uly = (window.getHeight() - kBoardDimension * kSquareSize)/2 + row * kSquareSize;
    window.setColor("White");
    window.fillRect(ulx, uly, kSquareSize, kSquareSize);
    window.setColor("Blue");
    window.drawRect(ulx, uly, kSquareSize, kSquareSize);
}

static void clearBoard(GWindow& window, Grid<bool>& board) {
	for (int row = 0; row < board.numRows(); row++) {
		for (int col = 0; col < board.numCols(); col++) {
			board[row][col] = false;
            drawSquare(window, row, col);
		}
	}
}

static void placeQueen(GWindow& window, int row, int col, string color) {
    drawSquare(window, row, col);
    int cx = (window.getWidth() - kBoardDimension * kSquareSize)/2 + col * kSquareSize + kSquareSize/2;
    int cy = (window.getHeight() - kBoardDimension * kSquareSize)/2 + row * kSquareSize + kSquareSize/2;
    window.setColor(color);
    window.drawString(kQueenLabel,
                      cx - window.getStringWidth(kQueenLabel)/2, cy + window.getFontAscent()/2);
    pause(kMillisecondDelay);
}

static bool isDirectionSafe(Grid<bool>& board, int row, int col, int drow, int dcol) {
	if (drow == 0 && dcol == 0) {
        return true;
    }
    
    row += drow;
	col += dcol;
	while (board.inBounds(row, col) && !board[row][col]) {
		row += drow;
		col += dcol;
	}
    
	return !board.inBounds(row, col);
}

static bool isSafe(Grid<bool>& board, int row, int col) {
	for (int drow  = -1; drow  <= 1; drow++) {
        if (!isDirectionSafe(board, row, col, drow, -1))
            return false;
	}
	
	return true;
}

static bool solve(GWindow& window, Grid<bool>& board, int col) {
    if (col == board.numCols()) return true;
    
    for (int rowToTry = 0; rowToTry < board.numRows(); rowToTry++) {
        placeQueen(window, rowToTry, col, kPossiblilityColor);
        if (isSafe(board, rowToTry, col)) {
            board[rowToTry][col] = true;
            placeQueen(window, rowToTry, col, kProvisionalColor);
            if (solve(window, board, col + 1)) {
                placeQueen(window, rowToTry, col, kPermanantColor);
                return true;
            }
            board[rowToTry][col] = false;
        }
        
        placeQueen(window, rowToTry, col, kNoPossibilityColor);
    }
    
    return false;
}

static void solve(GWindow& window, Grid<bool>& board) {
    solve(window, board, 0);
}

int main() {
    GWindow window(2 * kWindowDimension, kWindowDimension);
	Grid<bool> board(kBoardDimension, kBoardDimension);
    window.setWindowTitle("Solve " + integerToString(kBoardDimension) + " Queens");
    window.setFont("Courier-Bold-" + integerToString(0.7 * kSquareSize));
    clearBoard(window, board);
	solve(window, board);
    return 0;
}
